/*
 * Copyright (c) 2002-2012, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.pluginwizard.business.model;

import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.portal.service.spring.SpringContextService;

import java.util.Collection;


/**
 * This class provides instances management methods (create, find, ...) for PluginApplication objects
 */
public final class PluginApplicationHome
{
    // Static variable pointed at the DAO instance
    private static IPluginApplicationDAO _dao = (IPluginApplicationDAO) SpringContextService.getPluginBean( "pluginwizard",
            "pluginwizard.pluginApplicationDAO" );

    /**
     * Private constructor - this class need not be instantiated
     */
    private PluginApplicationHome(  )
    {
    }

    /**
     * Removes all the applications attached to a plugin
     * @param nIdPlugin The id of the plugin
     * @param plugin The plugin
     */
    public static void removePluginApplicationByPlugin( int nIdPlugin, Plugin plugin )
    {
        throw new UnsupportedOperationException( "Not yet implemented" ); //TODO
    }

    /**
     * Create an instance of the pluginApplication class
     * @param pluginApplication The instance of the PluginApplication which contains the informations to store
     * @param plugin the Plugin
     * @return The  instance of pluginApplication which has been created with its primary key.
     */
    public static PluginApplication create( PluginApplication pluginApplication, Plugin plugin )
    {
        _dao.insert( pluginApplication, plugin );

        return pluginApplication;
    }

    /**
     * Update of the pluginApplication which is specified in parameter
     * @param pluginApplication The instance of the PluginApplication which contains the data to store
     * @param plugin the Plugin
     * @return The instance of the  pluginApplication which has been updated
     */
    public static PluginApplication update( PluginApplication pluginApplication, Plugin plugin )
    {
        _dao.store( pluginApplication, plugin );

        return pluginApplication;
    }

    /**
     * Remove the pluginApplication whose identifier is specified in parameter
     * @param nPluginApplicationId The pluginApplication Id
     * @param plugin the Plugin
     */
    public static void remove( int nPluginApplicationId, Plugin plugin )
    {
        _dao.delete( nPluginApplicationId, plugin );
    }

    ///////////////////////////////////////////////////////////////////////////
    // Finders

    /**
     * Returns an instance of a pluginApplication whose identifier is specified in parameter
     * @param nKey The pluginApplication primary key
     * @param plugin the Plugin
     * @return an instance of PluginApplication
     */
    public static PluginApplication findByPrimaryKey( int nKey, Plugin plugin )
    {
        return _dao.load( nKey, plugin );
    }

    /**
     * Returns an instance of a pluginApplication whose identifier is specified in parameter
     * @param nKey The pluginApplication primary key
     * @param plugin the Plugin
     * @return an instance of PluginApplication
     */
    public static Collection<PluginApplication> findByPlugin( int nKey, Plugin plugin )
    {
        return _dao.selectByPlugin( nKey, plugin );
    }

    /**
     * Load the data of all the pluginApplication objects and returns them in form of a collection
     * @param plugin the Plugin
     * @return the collection which contains the data of all the pluginApplication objects
     */
    public static Collection<PluginApplication> getPluginApplicationsList( Plugin plugin )
    {
        return _dao.selectPluginApplicationsList( plugin );
    }
}
